'' =================================================================================================
''
''   File....... st_voltage_reading.spin
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2016 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated.... 23 APR 2016
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x                                     
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us


con { io pins }

  RX1     = 31 { I }                                            ' serial / programming
  TX1     = 30 { O }                                             
                                                                 
  SDA     = 29 { I }                                            ' i2c / eeprom
  SCL     = 28 { I/O }                                           
                                                                 
  ADC_CLK = 18 { O }                                            ' adc connections    
  ADC_CS  = 17 { O }
  ADC_DIO = 16 { I/O }                                           
  

con

  #(-1), ON, OFF                                                ' digital control


obj

' main                                                          ' * master Spin cog
  time : "jm_time"                                              '   timing and delays
  io   : "jm_io_basic"                                          '   essential io
  adc  : "jm_mcp3202_ez"                                        '   2-channel analog input
  term : "jm_fullduplexserial"                                  ' * serial IO for terminal
                                                                 
' * uses cog when loaded                                         
                                                                 
                                                                 
var

  long  adctimer
  long  adcidx
  long  adcraw[4]                                                           
                                                                 
                                                                 
pub main                                                         
                                                                 
  setup

  adctimer := time.millis                                       ' sync adc timer

  repeat
    process_voltage
    {
      run other scheduled processes
    }


pub process_voltage | avg, idx, hvolts

'' Reads voltage every 250ms
'' -- updates display with average of last 4 readings

  if ((time.millis - adctimer) < 250)                           ' if nottime to read 
    return                                                      '  abort

  adctimer += 250                                               ' set next update time
  adcraw[adcidx] := adc.read(0, adc#SE) << 2                    ' x4 for voltage divider
  if (++adcidx == 4)                                            ' advance adc index (with rollover)
    adcidx := 0

  avg := 0                                                                                                                  
  repeat idx from 0 to 3                                                                         
    avg += adcraw[idx]                                          ' accumulate last 4 readings                                                                          
  avg >>= 2                                                     ' average                        
  hvolts := avg ** $1F41_F420                                   ' x0.1221 (raw to 1/100 volts)

  display_voltage(avg, hvolts)   
                                                       
  
pub display_voltage(raw, volts)

'' Display raw and formatted voltage

  term.tx(term#HOME)
  term.str(string("Raw...... "))
  term.rjdec(raw, 4, " ")
  term.tx(13)
  term.str(string("Volts.... "))
  term.rjdec(volts / 100, 2, " ")
  term.tx(".")
  term.rjdec(volts // 100, 2, "0") 
                                                            
                                                                 
pub setup                                                        
                                                                 
'' Setup IO and objects for application                          
                                                                 
  time.start                                                    ' setup timing & delays
                                                                 
  io.start(0, 0)                                                ' clear all pins (master cog)

  adc.start(ADC_CS, ADC_CLK, ADC_DIO)                           ' connect to ADC
                                                                 
  term.start(RX1, TX1, %0000, 115_200)                          ' start serial for terminal *
                                                                 
                        
dat { license }                                                  

{{

  Copyright (C) 2016 Jon McPhalen

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}